@@ -172,7 +172,7 @@ static int _read_devs(struct dev_set *ds, const char *dir)
 	unsigned char dtype;
 	struct stat info;
 	char path[PATH_MAX];
-	dev_t dev;
+	dev_t dev = { 0 };
 	int r = 1;
 
         if (!(dr = opendir(dir))) {
@@ -194,8 +194,10 @@ static int _read_devs(struct dev_set *ds, const char *dir)
 		dtype = d->d_type;
 
 		if (dtype == DT_UNKNOWN) {
-			if (stat(path, &info) >= 0) {
-				if (S_ISDIR(info.st_mode))
+			if (lstat(path, &info) >= 0) {
+				if (S_ISLNK(info.st_mode))
+					dtype = DT_LNK;
+				else if (S_ISDIR(info.st_mode))
 					dtype = DT_DIR;
 				else if (S_ISREG(info.st_mode))
 					dtype = DT_REG;
